==> support.aas-0.5.0-sources.jar: de/iip_ecosphere/platform/support/aas/Submodel.java <==
/**
 * ******************************************************************************
 * Copyright (c) {2020} The original author or authors
 *
 * All rights reserved. This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License 2.0 which is available 
 * at http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
 * which is available at https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0 OR EPL-2.0
 ********************************************************************************/

package de.iip_ecosphere.platform.support.aas;

import de.iip_ecosphere.platform.support.Builder;
import de.iip_ecosphere.platform.support.aas.SubmodelElementCollection.SubmodelElementCollectionBuilder;

/**
 * Represents an AAS sub-model.
 * 
 * @author Holger Eichelberger, SSE
*/
public interface Submodel extends Element, HasSemantics, Identifiable, Qualifiable, HasDataSpecification, HasKind, 
    ElementContainer, DeferredParent, ElementsAccess {

    /**
     * Encapsulated logic to build a sub-model.
     * 
     * @author Holger Eichelberger, SSE
     */
    public interface SubmodelBuilder extends SubmodelElementContainerBuilder, DeferredBuilder<Submodel> {
        
        /**
         * Creates a reference on the sub-model under construction.
         * 
         * @return the reference
         */
        public Reference createReference();

    }

    /**
     * Returns a sub-model element with the given name.
     * 
     * @param idShort the short id of the property
     * @return the sub-model element, <b>null</b> for none
     */
    public SubmodelElement getSubmodelElement(String idShort);

    /**
     * Returns a sub-model elements collection builder either by providing access to an existing collection or through 
     * a builder to add a new sub-model elements collection (ultimately only if {@link Builder#build()} was called).
     * 
     * @param ordered whether the collection shall be ordered or not
     * @param allowDuplicates whether the collection allows duplicates or not
     * @param idShort the short id of the sub-model
     * @return the sub-model collection builder
     */
    public SubmodelElementCollectionBuilder createSubmodelElementCollectionBuilder(String idShort, boolean ordered, 
        boolean allowDuplicates);

    /**
     * Returns the reference to the AAS.
     * 
     * @return the reference
     */
    public Reference createReference();

    /**
     * Deletes a sub-model element.
     * 
     * @param elt the element to delete
     */
    public void delete(SubmodelElement elt);

}

==> support.aas-0.6.0-sources.jar: de/iip_ecosphere/platform/support/aas/Submodel.java <==
/**
 * ******************************************************************************
 * Copyright (c) {2020} The original author or authors
 *
 * All rights reserved. This program and the accompanying materials are made 
 * available under the terms of the Eclipse Public License 2.0 which is available 
 * at http://www.eclipse.org/legal/epl-2.0, or the Apache License, Version 2.0
 * which is available at https://www.apache.org/licenses/LICENSE-2.0.
 *
 * SPDX-License-Identifier: Apache-2.0 OR EPL-2.0
 ********************************************************************************/

package de.iip_ecosphere.platform.support.aas;

import java.util.function.Consumer;

import de.iip_ecosphere.platform.support.Builder;
import de.iip_ecosphere.platform.support.aas.SubmodelElementCollection.SubmodelElementCollectionBuilder;

/**
 * Represents an AAS sub-model.
 * 
 * @author Holger Eichelberger, SSE
*/
public interface Submodel extends Element, HasSemantics, Identifiable, Qualifiable, HasDataSpecification, HasKind, 
    ElementContainer, DeferredParent, ElementsAccess {

    /**
     * Encapsulated logic to build a sub-model.
     * 
     * @author Holger Eichelberger, SSE
     */
    public interface SubmodelBuilder extends SubmodelElementContainerBuilder, DeferredBuilder<Submodel> {
        
        /**
         * Creates a reference on the sub-model under construction.
         * 
         * @return the reference
         */
        public Reference createReference();

    }

    /**
     * Returns a sub-model element with the given name.
     * 
     * @param idShort the short id of the property
     * @return the sub-model element, <b>null</b> for none
     */
    public SubmodelElement getSubmodelElement(String idShort);

    /**
     * Returns a sub-model elements collection builder either by providing access to an existing collection or through 
     * a builder to add a new sub-model elements collection (ultimately only if {@link Builder#build()} was called).
     * 
     * @param ordered whether the collection shall be ordered or not
     * @param allowDuplicates whether the collection allows duplicates or not
     * @param idShort the short id of the sub-model
     * @return the sub-model collection builder
     */
    public SubmodelElementCollectionBuilder createSubmodelElementCollectionBuilder(String idShort, boolean ordered, 
        boolean allowDuplicates);

    /**
     * Returns the reference to the AAS.
     * 
     * @return the reference
     */
    public Reference createReference();

    /**
     * Interprets {@code path} to end with a contained submodel elements collection and applies
     * {@code func} to it without loading/caching the submodel into its interface representation.
     * 
     * @param func the function to apply for creation
     * @param propagate the change into the interface instance; if applied frequently, may imply a performance issue
     * @param path the path to the submodel
     * @return {@code true} if something has been applied, {@code false} else
     */
    public boolean create(Consumer<SubmodelElementContainerBuilder> func, boolean propagate, String... path);

    @FunctionalInterface
    public interface IteratorFunction<T extends SubmodelElement> {

        /**
         * Applies the function.
         * 
         * @param element the element to apply the function to
         * @return {@code true} for continuing the iteration, {@code false} for stopping
         */
        public boolean apply(T element);
        
    }
    
    /**
     * Interprets {@code path} to end with a submodel element or a submode elements collection and applies
     * {@code func} to the element or to all elements in the collection without loading/caching the submodel 
     * elements into its interface representation.
     * 
     * @param <T> type of elements
     * @param func the function to apply
     * @param cls class of elements to filter during the iteration
     * @param path the path to the submodel element/Collection
     * @return {@code true} if something has been applied, {@code false} else
     */
    public <T extends SubmodelElement> boolean iterate(IteratorFunction<T> func, Class<T> cls, String... path);    

}
